@@ -487,12 +487,15 @@ def main(args):
         else:
             sample_prefixes = [str(item) for item in range(1, len(sfiles)+1)]
         if len(sfiles) != len(sample_prefixes):
-            common.MSG("--sample-prefixes must be same length as --samples")
+            common.WARNING("--sample-prefixes must be same length as --samples")
             return 1
         for sf in sfiles:
             sample_list = np.array(
                 [item.strip() for item in open(sf, "r").readlines()]
             )
+            if not np.any(np.isin(all_samples, sample_list)):
+                common.WARNING("No samples from {} found in the VCF file".format(sf))
+                return 1
             sample_indexes.append(np.isin(all_samples, sample_list))
     else:
         sample_indexes = [None] # None is used to mean all samples
@@ -513,7 +516,7 @@ def main(args):
     try:
         if args.out == "stdout":
             if args.plot_afreq:
-                common.MSG("Cannot use --out stdout when generating plots")
+                common.WARNING("Cannot use --out stdout when generating plots")
                 return 1
             outf = sys.stdout
         else:
